# - *- coding: utf- 8 - *-

from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.Standby import TryQuitMainloop

from Components.ActionMap import ActionMap
from Components.Sources.StaticText import StaticText
from Components.config import config, getConfigListEntry, ConfigSubsection, ConfigSelection, ConfigYesNo, ConfigDescription, fileExists
from Components.ConfigList import ConfigList, ConfigListScreen

#============ Version =============
version = "V1.0"

#============ check installed Plugins =============
isCoolTVInstalled = fileExists("/usr/lib/enigma2/python/Plugins/Extensions/CoolTVGuide/plugin.pyo")
isTMDbInstalled = fileExists('/usr/lib/enigma2/python/Plugins/Extensions/tmdb/plugin.pyo')
isAELInstalled = fileExists('/usr/lib/enigma2/python/Plugins/Extensions/AdvancedEventLibrary/AdvancedEventLibrarySystem.pyo') and fileExists('/usr/lib/enigma2/python/Tools/AdvancedEventLibrary.pyo')
isSerienrecorderInstalled = fileExists('/usr/lib/enigma2/python/Plugins/Extensions/serienrecorder/plugin.pyo')
isAutoTimerInstalled = fileExists('/usr/lib/enigma2/python/Plugins/Extensions/AutoTimer/plugin.pyo')

#============ config =============
choiceList_Keys = [
	(False, _('---')),
	('RedLong', _('Red-Long')),
	('Green', _('Green')),
	('GreenLong', _('Green-Long')),
	('Yellow', _('Yellow')),
	('YellowLong', _('Yellow-Long')),
	('Blue', _('Blue')),
	('BlueLong', _('Blue-Long')),
	('OK', _('OK')),
	('EPG', _('EPG/Info')),
	('EPGLong', _('EPG/Info-Long')),
	('TV', _('TV')),
]
choiceList_MoveContent = [
	(False, _('---')),
	('1', _('+1')),
	('2', _('+2')),
	('3', _('+3')),	
	('4', _('+4')),
	('5', _('+5')),
	('6', _('+6')),
	('7', _('+7')),
	('8', _('+8')),
	('9', _('+9')),
	('10', _('+10')),
	('11', _('+11')),
	('12', _('+12')),
	('13', _('+13')),
	('14', _('+14')),
	('15', _('+15')),
	('-15', _('-15')),
	('-14', _('-14')),
	('-13', _('+13')),
	('-12', _('-12')),
	('-11', _('-11')),
	('-10', _('-10')),
	('-9', _('-9')),
	('-8', _('-8')),
	('-7', _('-7')),
	('-6', _('-6')),
	('-5', _('-5')),
	('-4', _('-4')),
	('-3', _('-3')),
	('-2', _('-2')),
	('-1', _('-1')),
]
choiceList_Disabled = [
	(False, _('---')),
]

config.plugins.CoolExtra = ConfigSubsection()

#Tastenzuordnung / CoolInfoBox
desc_key = _("Soll die Integration auch in CoolSearch erfolgen, kann nur OK, EPG/Info, EPG/Info-Long oder TV verwendet werden")
desc_cib = _("Plugin zur CoolInfoBox hinzufügen")

if isCoolTVInstalled and isTMDbInstalled:
	config.plugins.CoolExtra.key_tmdb = ConfigSelection(default=False, choices=choiceList_Keys)
	config.plugins.CoolExtra.cib_tmdb = ConfigYesNo(default=False)
	desc_key_tmdb = desc_key
	desc_cib_tmdb = desc_cib
else:
	config.plugins.CoolExtra.key_tmdb = ConfigSelection(default=False, choices=choiceList_Disabled)
	config.plugins.CoolExtra.cib_tmdb = ConfigSelection(default=False, choices=choiceList_Disabled)
	desc_key_tmdb = _("Plugin TMDb nicht installiert")
	desc_cib_tmdb = _("Plugin TMDb nicht installiert")

if isCoolTVInstalled and isAELInstalled:
	config.plugins.CoolExtra.key_aeleditor = ConfigSelection(default=False, choices=choiceList_Keys)
	config.plugins.CoolExtra.cib_aeleditor = ConfigYesNo(default=False)
	desc_key_aeleditor = desc_key
	desc_cib_aeleditor = desc_cib
else:
	config.plugins.CoolExtra.key_aeleditor = ConfigSelection(default=False, choices=choiceList_Disabled)
	config.plugins.CoolExtra.cib_aeleditor = ConfigSelection(default=False, choices=choiceList_Disabled)
	desc_key_aeleditor = _("Plugin Advanced-Event-Library nicht installiert")
	desc_cib_aeleditor = _("Plugin Advanced-Event-Library nicht installiert")
	
if isCoolTVInstalled and isSerienrecorderInstalled:
	config.plugins.CoolExtra.key_serienrecorder = ConfigSelection(default=False, choices=choiceList_Keys)
	config.plugins.CoolExtra.cib_serienrecorder = ConfigYesNo(default=False)
	desc_key_serienrecorder = desc_key
	desc_cib_serienrecorder = desc_cib
else:
	config.plugins.CoolExtra.key_serienrecorder = ConfigSelection(default=False, choices=choiceList_Disabled)
	config.plugins.CoolExtra.cib_serienrecorder = ConfigSelection(default=False, choices=choiceList_Disabled)
	desc_key_serienrecorder = _("Plugin Serienrecorder nicht installiert")
	desc_cib_serienrecorder = _("Plugin Serienrecorder nicht installiert")
	
if isCoolTVInstalled and isAutoTimerInstalled:
	config.plugins.CoolExtra.cib_autotimer = ConfigYesNo(default=False)
	desc_cib_autotimer = desc_cib
else:
	config.plugins.CoolExtra.cib_autotimer = ConfigSelection(default=False, choices=choiceList_Disabled)
	desc_cib_autotimer = _("Plugin AutoTimer nicht installiert")
	
config.plugins.CoolExtra.cib_coolextra = ConfigYesNo(default=False)

config.plugins.CoolExtra.pvr_action = ConfigSelection(default=False, choices=[(False, _('---')),('MovieSelection', _('Filmliste bevorzugt'))])
config.plugins.CoolExtra.power_action = ConfigSelection(default=False, choices=[(False, _('---')),('ShutDownMenu', _('Ausschaltmenü')),('Standby', _('Standby'))])

#CoolTVGuide-Anpassungen
config.plugins.CoolExtra.tvg_center_picon = ConfigYesNo(default=False)
config.plugins.CoolExtra.tvg_unlimit_picon_rowheight = ConfigYesNo(default=False)
config.plugins.CoolExtra.tvg_border_picon = ConfigYesNo(default=False)
config.plugins.CoolExtra.tvg_scale_rowbackground = ConfigYesNo(default=False)

#CoolInfoGuide-Anpassungen
config.plugins.CoolExtra.ig_next_move_content = ConfigSelection(default=False, choices=choiceList_MoveContent)

#Sonstiges
config.plugins.CoolExtra.logging = ConfigYesNo(default=False)

#============ CoolExtraConfigScreen =============
class CoolExtraConfigScreen(Screen, ConfigListScreen):
	def __init__(self, session):
		Screen.__init__(self, session)		
		self.skinName = [ "CoolExtraConfigScreen", "Setup" ]
		self.setTitle(_("CoolExtra - Einstellungen") + " " + version)
		
		self.list = []
		self.buildConfigList()
		ConfigListScreen.__init__(self, self.list, session = session, on_change = self.changedEntry)
		
		self["config"].onSelectionChanged.append(self.updateDesc)
		
		self["actions"] = ActionMap(["ColorActions", "SetupActions"],
			{
				"cancel": self.keyCancel,
				"save": self.keyOK,
				"red": self.keyCancel,
				"green": self.keyGreen,
			}, -2)

		self["key_green"] = StaticText(_("Save"))
		self["key_red"] = StaticText(_("Cancel"))

	def buildConfigList(self):
		self.list = []
		
		if isCoolTVInstalled:
			#Page Tastenzuordnung
			self.list.append(getConfigListEntry(_("Tastenzuordnung"), ConfigDescription()))
			self.list.append(getConfigListEntry(_("TMDb-Plugin (Taste)"), config.plugins.CoolExtra.key_tmdb, desc_key_tmdb))
			self.list.append(getConfigListEntry(_("AEL-Editor (Taste)"), config.plugins.CoolExtra.key_aeleditor, desc_key_aeleditor))
			self.list.append(getConfigListEntry(_("Serienrecorder-Plugin (Taste)"), config.plugins.CoolExtra.key_serienrecorder, desc_key_serienrecorder))
			
			self.list.append(getConfigListEntry(_("Aktion: PVR/Video-Taste"), config.plugins.CoolExtra.pvr_action, _("Aktion beim Drücken der PVR/Video-Taste (Standard: Enhanced-Movie-Center, wenn installiert)")))
			self.list.append(getConfigListEntry(_("Aktion: Power-Taste"), config.plugins.CoolExtra.power_action, _("Aktion beim Drücken der Power-Taste (Standard: Ausschalt-Timer)")))
			
			#Page CoolInfoBox
			self.list.append(getConfigListEntry(_("CoolInfoBox"), ConfigDescription()))
			self.list.append(getConfigListEntry(_("TMDb-Plugin (CoolInfoBox)"), config.plugins.CoolExtra.cib_tmdb, desc_cib_tmdb))
			self.list.append(getConfigListEntry(_("AEL-Editor (CoolInfoBox)"), config.plugins.CoolExtra.cib_aeleditor, desc_cib_aeleditor))
			self.list.append(getConfigListEntry(_("Serienrecorder-Plugin (CoolInfoBox)"), config.plugins.CoolExtra.cib_serienrecorder, desc_cib_serienrecorder))
			self.list.append(getConfigListEntry(_("AutoTimer-Plugin (CoolInfoBox)"), config.plugins.CoolExtra.cib_autotimer, desc_cib_autotimer))
			self.list.append(getConfigListEntry(_("CoolExtra - Einstellungen (CoolInfoBox)"), config.plugins.CoolExtra.cib_coolextra, desc_cib))
			
			#Page CoolTVGuide
			self.list.append(getConfigListEntry(_("CoolTVGuide"), ConfigDescription()))
			self.list.append(getConfigListEntry(_("Picons horizontal zentrieren"), config.plugins.CoolExtra.tvg_center_picon, _("Picons in den CoolTVGuide-Ansichten (TV, Multi, Tiny, Nice) zentrieren")))
			self.list.append(getConfigListEntry(_("Rahmen um Picons zeichnen "), config.plugins.CoolExtra.tvg_border_picon, _("Rahmen um Picons in den CoolTVGuide-Ansichten (TV, Multi, Tiny, Nice) zeichnen")))
			self.list.append(getConfigListEntry(_("Limit Picon-Zeilenhöhe deaktivieren"), config.plugins.CoolExtra.tvg_unlimit_picon_rowheight, _("Deaktiviert das Limit von 80 zum Vergrössern der Zeilenhöhe von CoolTVGuide und CoolMultiGuide")))
			self.list.append(getConfigListEntry(_("Zeilen-Hintergründe skalieren"), config.plugins.CoolExtra.tvg_scale_rowbackground, _("Nicht für alle Hintergründe geeignet und nur sinnvoll, wenn die Hintergrundgrafik kleiner ist als die Zeilenhöhe")))
			
			#Page CoolInfoGuide
			self.list.append(getConfigListEntry(_("CoolInfoGuide"), ConfigDescription()))
			self.list.append(getConfigListEntry(_("NextEvents: Zeileninhalt vertikal verschieben (+/-)"), config.plugins.CoolExtra.ig_next_move_content, _("Ermöglicht das Zentrieren des Zeileninhalts der NextEvents-Liste")))
			
			#Page Sonstiges
			self.list.append(getConfigListEntry(_("Sonstiges"), ConfigDescription()))
			self.list.append(getConfigListEntry(_("Schreibe Logdatei"), config.plugins.CoolExtra.logging, _("Logdatei: /tmp/CoolExtra.log")))
		
		else:
			self.list.append(getConfigListEntry(_("=== Plugin CoolTVGuide V7.7 OE2.0 nicht installiert ==="), ConfigDescription()))

	def changedEntry(self):
		pass
	
	def updateDesc(self):
		cur = self["config"].getCurrent()
		if cur and len(cur)>2:
			self["configdesc"].text = cur[2]
	
	def keyOK(self):
		return
		#self.keyGreen()

	def keyGreen(self):
		for x in self["config"].list:
			x[1].save()
		#Restart GUI
		msg = self.session.openWithCallback(self.restartGUI,MessageBox,_("Restart necessary, restart GUI now?"), MessageBox.TYPE_YESNO)
		msg.setTitle(_("Message"))
		
	def restartGUI(self,answer):		
		if answer:
			self.session.open(TryQuitMainloop, 3)
		else:
			self.close()
		