# - *- coding: utf- 8 - *-
from Plugins.Plugin import PluginDescriptor
from Components.config import config, ConfigSelection, ConfigInteger, fileExists
from Screens.ChoiceBox import ChoiceBox
from Screens.InfoBar import InfoBar
from Screens.Menu import MainMenu, mdom
import Screens.Standby
from enigma import eListboxPythonMultiContent
from functools import partial
import time
import os

from .CoolExtraConfig import CoolExtraConfigScreen, isCoolTVInstalled, isTMDbInstalled, isAELInstalled, isSerienrecorderInstalled, isAutoTimerInstalled

if isCoolTVInstalled:
	import Plugins.Extensions.CoolTVGuide.plugin as coolTV	
if isTMDbInstalled:
	from Plugins.Extensions.tmdb.tmdb import tmdbScreen
if isAELInstalled:
	from Plugins.Extensions.AdvancedEventLibrary.AdvancedEventLibrarySystem import Editor as aelEditor
	from Tools.AdvancedEventLibrary import convertTitle, convertTitle2, convert2base64, previewImages
if isSerienrecorderInstalled:
	from Plugins.Extensions.serienrecorder.SerienRecorderSearchResultScreen import serienRecSearchResultScreen 
	from Plugins.Extensions.serienrecorder.SerienRecorderMarkerScreen import serienRecMarker
if isAutoTimerInstalled:
	from Plugins.Extensions.AutoTimer.AutoTimerEditor import addAutotimerFromEvent
	from Plugins.Extensions.AutoTimer.plugin import main as autoTimerSafe
	
coolInfoBoxList = []
tvg_center_picon = False
tvg_border_picon = False
tvg_scale_rowbackground = False
ig_next_move_content = False
logFile = "/tmp/CoolExtra.log"
logging = False

#============= Cool-Codes ==================
coolScreens = {
	"CoolInfoGuide" : "zDKhcfAp",
	"CoolSingleGuide" : "XEArkQXQCQRMEL",
	"CoolEasyGuide" : "sduqcURd",
	"CoolChannelGuide" : "fECRrOPDKlFDx",
	"CoolTVGuide" : "znjhVZLcrBZZBbq",
	"CoolSearch" : "lyqBhZ"
}
		
coolKeys = {
	"Red" : ("GwhJbNuquP","C12"),
	"RedLong" : ("fbUws","C13"),
	"Green" : ("gXppchsrp","C14"),
	"GreenLong" : ("mzzBQRAohy","C15"),
	"Yellow" : ("bKpKe","C16"),
	"YellowLong" : ("qOlhOHWiFHpIC","C17"),
	"Blue" : ("qsDCwRsYOveEFy","C18"),
	"BlueLong" : ("uHqjdBJMLj","C19"),
	"OK" : ("WhRkmBFDyxfV","C20"),
	"EPG" : ("NxrXAek","C22"),
	"EPGLong" : ("uWIjFbGhtX","C23"),
	"TV" : ("EekrcnjXd","-"),
}
coolInfoBoxFunc = "fxVcQUikA"
coolPVRFunc = "nePylQgKlghpaVP"
coolEventInfoFunc = "JvSwzXgDeSH"
coolPowerFunc = "EUtMyA"
coolTVGuideTable_Class = "rHAPXA"
coolTVGuideTable_BuildFunc = "KIiMbYLKzL"
coolTVGuideTable_bordercolL = "mFuSMdHDOvptbxd"
coolInfoGuideNext_Class = "TyJjFMDpxnnJch"
coolInfoGuideNext_BuildFunc = "zKQMKGEg"
# =============================================

def writeLog(logText):
	if not logging: return
	t = time.localtime()
	logTime = time.strftime("%d.%m.%Y %H:%M:%S: ",t)
	log = open(logFile,"a")
	log.write(str(logTime) + str(logText) + "\n")
	log.close()

def main(session, **kwargs):
	session.open(CoolExtraConfigScreen)
	
def sessionstart(reason, **kwargs):
	if reason == 0 and isCoolTVInstalled:
		
		#========== Logging =============
		if fileExists(logFile): os.remove(logFile)
		global logging
		logging = config.plugins.CoolExtra.logging
		writeLog("======== sessionstart ========")
		
		#========== Add Plugin to Key =============
		pluginKey = config.plugins.CoolExtra.key_tmdb.value
		if pluginKey and isTMDbInstalled:
			writeLog("Add TMDB to Key: " + pluginKey)
			addPluginToKey(pluginKey, openTMDb, "TMDb", _("TMDb"))
		
		pluginKey = config.plugins.CoolExtra.key_aeleditor.value
		if pluginKey and isAELInstalled:
			writeLog("Add AEL-Editor to Key: " + pluginKey)
			addPluginToKey(pluginKey, openAELEditor, "AEL-Editor", _("AEL-Editor"))
		
		pluginKey = config.plugins.CoolExtra.key_serienrecorder.value
		if pluginKey and isSerienrecorderInstalled:
			writeLog("Add Serienrecorder to Key: " + pluginKey)
			addPluginToKey(pluginKey, openSerienRecorder, "Serienrecorder",_("Serienrecorder"))
		
		#========== Add Plugin to CoolInfoBox =============
		cib_TMDb = config.plugins.CoolExtra.cib_tmdb.value
		cib_AELEditor = config.plugins.CoolExtra.cib_aeleditor.value
		cib_Serienrecorder = config.plugins.CoolExtra.cib_serienrecorder.value
		cib_AutoTimer = config.plugins.CoolExtra.cib_autotimer.value
		cib_CoolExtra = config.plugins.CoolExtra.cib_coolextra.value
		
		if cib_TMDb:
			writeLog("Add TMDb to CoolInfoBox")
			coolInfoBoxList.append((_('TMDb'), 'TMDb'))
		
		if cib_AELEditor:
			writeLog("Add AEL-Editor to CoolInfoBox")
			coolInfoBoxList.append((_('AEL-Editor'), 'AEL-Editor'))
		
		if cib_Serienrecorder:
			writeLog("Add Serienrecorder to CoolInfoBox")
			coolInfoBoxList.append((_('Serienrecorder'), 'Serienrecorder'))
			
		if cib_AutoTimer:
			writeLog("Add AutoTimer to CoolInfoBox")
			coolInfoBoxList.append((_('AutoTimer'), 'AutoTimer'))
		
		if cib_CoolExtra:
			writeLog("Add CoolExtra-Config to CoolInfoBox")
			coolInfoBoxList.append((_('CoolExtra - Einstellungen'), 'CoolExtraConfig'))
		
		if len(coolInfoBoxList) > 0:
			initCoolInfoBox()
		
		#========= PVR-Action ===========
		pvr_action = config.plugins.CoolExtra.pvr_action.value
		if pvr_action and pvr_action == "MovieSelection":
			if hasattr(coolTV,coolPVRFunc):
				setattr(coolTV, coolPVRFunc, openMovieSelection)
				writeLog("Global PVR-Action set to MovieSelection")
			else:
				writeLog("+++ ERROR PVR-Action: KeyFunc - " + coolPVRFunc + " not found")
		
		#========= Power-Action ===========
		power_action = config.plugins.CoolExtra.power_action.value
		if power_action:
			if power_action == "ShutDownMenu":
				writeLog("Set Power-Action to ShutDownMenu")
				initPowerActiom(powerActionShutdownMenu)
			elif power_action == "Standby":
				writeLog("Set Power-Action to Standby")
				initPowerActiom(powerActionStandby)
		
		#======= TVG (center picon, border picon, scale row background) ======
		global tvg_center_picon
		global tvg_border_picon
		global tvg_scale_rowbackground
		tvg_center_picon = config.plugins.CoolExtra.tvg_center_picon.value
		tvg_border_picon = config.plugins.CoolExtra.tvg_border_picon.value
		tvg_scale_rowbackground = config.plugins.CoolExtra.tvg_scale_rowbackground.value
		if tvg_center_picon or tvg_border_picon or tvg_scale_rowbackground:
			if tvg_center_picon: 
				writeLog("Activate: CoolTVGuide - Center-Picon")
			if tvg_border_picon: 
				writeLog("Activate: CoolTVGuide - Border-Picon")
			if tvg_scale_rowbackground: 
				writeLog("Activate: CoolTVGuide - Scale row background")
			if hasattr(coolTV,coolTVGuideTable_Class):
				getattr(coolTV, coolTVGuideTable_Class).GUIcreate = newTVGuideTable_GUIcreate
			else:
				writeLog("+++ ERROR Adjust-CoolTVGuide: Class - " + coolTVGuideTable_Class + " not found")
				
		#=== TVG (unlimit Picon-RowHeight: C50 and Picon-Height: C49) ====
		if config.plugins.CoolExtra.tvg_unlimit_picon_rowheight.value:
			writeLog("Deactivate: CoolTVGuide - Limit Picon-Row-Height and Picon-Height")
			coolTV.config.CTVG.C50 = ConfigInteger(default=60, limits=(30,999))
			coolTV.config.CTVG.C49 = ConfigInteger(default=60, limits=(30,999))
		
		#======= IG (increase top dustance) ======
		global ig_next_move_content
		ig_next_move_content = config.plugins.CoolExtra.ig_next_move_content.value
		if ig_next_move_content:
			writeLog("Activate: CoolInfoGuide - NextEvents move content (" + ig_next_move_content + ")")
			if hasattr(coolTV,coolInfoGuideNext_Class):
				getattr(coolTV, coolInfoGuideNext_Class).GUIcreate = newInfoGuideNext_GUIcreate
			else:
				writeLog("+++ ERROR Adjust-CoolInfoGuide: Class - " + coolInfoGuideNext_Class + " not found")	
			
		writeLog("======== Adjustments finished ========")
		

#========== newTVGuide-Table-Functions =============		
def newTVGuideTable_GUIcreate(self, parent):
	self.instance = self.createWidget(parent)
	self.l.setBuildFunc(partial(newTVGuideTable_BuildFunc,self))
	self.postWidgetCreate(self.instance)
	
def newTVGuideTable_BuildFunc(self,serviceRef,serviceName,epgData):
	itemRow = [None]
	try:
		if not hasattr(self,coolTVGuideTable_BuildFunc):
			writeLog("+++ ERROR newTVGuideTable_BuildFunc: Func - " + coolTVGuideTable_BuildFunc + " not found")
			return itemRow
		
		itemRow = getattr(self,coolTVGuideTable_BuildFunc)(serviceRef,serviceName,epgData)
		if not itemRow or len(itemRow) == 0:
			writeLog("+++ ERROR newTVGuideTable_BuildFunc: NoData")
			return itemRow
		
		itemBkPicon = None	
		for index,item in enumerate(itemRow): # List starts with None
			#Check if Picon in row (PixmapType=ALPHABLEND)
			if item and len(item) == 9 and item[0] == eListboxPythonMultiContent.TYPE_PIXMAP_ALPHABLEND:
				x,y = item[1],item[2]
				w,h = item[3],item[4]
				png = item[5]
				bkColor = item[6]
				
				#Center-Picon
				if tvg_center_picon:
					ph = (h+2) - 2*y #picon height
					pw = ph / 0.6
					new_x = int((w+2-pw) / 2) + 1 #+1=rounding-cor.
					itemRow[index] = (
						eListboxPythonMultiContent.TYPE_PIXMAP_ALPHABLEND, new_x, y, w, h, png, None, None, 0)
						
				#Border-Picon
				bkPicon_Border_Width = bkPicon_Border_Color = None
				if tvg_border_picon:
					if hasattr(self,coolTVGuideTable_bordercolL):
						bkPicon_Border_Width = 1
						bkPicon_Border_Color = getattr(self,coolTVGuideTable_bordercolL)
						
				#Additional Picon Background
				itemBkPicon = (eListboxPythonMultiContent.TYPE_TEXT, 0, 0, w+3, h+2, 0, 0, "", None, None, bkColor, bkColor, bkPicon_Border_Width, bkPicon_Border_Color)
				
				#break
				if not tvg_scale_rowbackground: break
			
			#Scale Backgrounds
			if tvg_scale_rowbackground and item and len(item) == 9 and item[0] == eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST:
				itemRow[index] = item[:8] + (4,) #BT_SCALE
				
		#Add additional Picon-Background to List before Picon
		if itemBkPicon:
			itemRow.insert(1,itemBkPicon)
				
	except Exception as ex:
		writeLog("+++ ERROR newTVGuideTable_BuildFunc: " + str(ex))
		
	return itemRow
	

#========== CoolInfoGuide-NextEvents-Functions =============		
def newInfoGuideNext_GUIcreate(self, parent):
	self.instance = self.createWidget(parent)
	self.l.setBuildFunc(partial(newInfoGuideNext_BuildFunc,self))
	self.postWidgetCreate(self.instance)
	
def newInfoGuideNext_BuildFunc(self,ref,eventId,starttime,duration,eventName):
	itemRow = [None]
	try:
		if not hasattr(self,coolInfoGuideNext_BuildFunc):
			writeLog("+++ ERROR newInfoGuideNext_BuildFunc: Func - " + coolInfoGuideNext_BuildFunc + " not found")
			return itemRow
		
		itemRow = getattr(self,coolInfoGuideNext_BuildFunc)(ref,eventId,starttime,duration,eventName)
		if not itemRow or len(itemRow) == 0:
			writeLog("+++ ERROR newInfoGuideNext_BuildFunc: NoData")
			return itemRow
		
		for index,item in enumerate(itemRow): # List starts with None
			if item and len(item) >= 3 and item[0] == eListboxPythonMultiContent.TYPE_TEXT:
				x,y = item[1],item[2]			
				new_y = int(ig_next_move_content)
				itemRow[index] = item[:2] + (new_y,) + item[3:]
							
	except Exception as ex:
		writeLog("+++ ERROR newInfoGuideNext_BuildFunc: " + str(ex))
		
	return itemRow

#========== Add/Init-Functions (Keys/CoolInfoBox) =============
def addPluginToKey(pluginKey,newKeyFunc,newKeyID,newKeyText):
	try:
		keyFunc,keyConfig = coolKeys.get(pluginKey,(None,None))
		if not keyFunc or not keyConfig: return
		for coolScreenName, coolScreenCode in coolScreens.iteritems():
			#skip CoolSearch if colored Buttons
			if coolScreenName == "CoolSearch": 
				if "Red" in pluginKey or "Green" in pluginKey or "Yellow" in pluginKey or "Blue" in pluginKey:	
					continue
			#skip CoolEasyGuide if Opt. timeJump {yellow/blue}
			if coolScreenName == "CoolEasyGuide" and config.CTVG.C57.value and (pluginKey == "Yellow" or pluginKey == "Blue"):
				continue
			if not hasattr(coolTV,coolScreenCode): 
				writeLog("+++ ERROR addPluginToKey: Screen - " + coolScreenName + " not found")
				continue
			coolScreen = getattr(coolTV, coolScreenCode)
			if not hasattr(coolScreen,keyFunc): 
				writeLog("+++ ERROR addPluginToKey: KeyFunc - " + keyFunc + " not found")
				continue
			setattr(coolScreen, keyFunc, newKeyFunc)
			writeLog("New Key-Function for Key:" + pluginKey + " added to Screen: " + coolScreenName)
		if hasattr(config.CTVG,keyConfig):
			setattr(config.CTVG, keyConfig, ConfigSelection(default=newKeyID, choices=[(newKeyID, newKeyText)]))	
	except Exception as ex:
		writeLog("+++ ERROR addPluginToKey: " + str(ex))
		
def initCoolInfoBox():
	try:
		for coolScreenName, coolScreenCode in coolScreens.iteritems():
			if coolScreenName == "CoolSearch": continue
			if not hasattr(coolTV,coolScreenCode): 
				writeLog("+++ ERROR initCoolInfoBox: Screen - " + coolScreenName + " not found") 
				continue
			coolScreen = getattr(coolTV, coolScreenCode)
			if not hasattr(coolScreen,coolInfoBoxFunc): 
				writeLog("+++ ERROR initCoolInfoBox: CIBFunc - " + coolInfoBoxFunc + " not found") 
				continue
			setattr(coolScreen, coolInfoBoxFunc, showCoolInfoBox)
	except Exception as ex:
		writeLog("+++ ERROR initCoolInfoBox: " + str(ex))
		
def initPowerActiom(newPowerFunc):
	try:
		for coolScreenName, coolScreenCode in coolScreens.iteritems():
			if not hasattr(coolTV,coolScreenCode): 
				writeLog("+++ ERROR initPowerActiom: Screen - " + coolScreenName + " not found") 
				continue
			coolScreen = getattr(coolTV, coolScreenCode)
			if not hasattr(coolScreen,coolPowerFunc): 
				writeLog("+++ ERROR initPowerActiom: PowerFunc - " + coolPowerFunc + " not found") 
				continue
			setattr(coolScreen, coolPowerFunc, newPowerFunc)
			writeLog("New Power-Action added to Screen: " + coolScreenName)
	except Exception as ex:
		writeLog("+++ ERROR initPowerActiom: " + str(ex))

#========== Actions =============
def showCoolInfoBox(self):
	self.session.openWithCallback(partial(cbCoolInfoBox,self), ChoiceBox, title=_('CoolInfoBox'), list=coolInfoBoxList)

def cbCoolInfoBox(self, sel=None):
	sel = sel and sel[1]
	if sel == "TMDb":
		openTMDb(self)
	if sel == "AEL-Editor":
		openAELEditor(self)
	if sel == "Serienrecorder":	
		openSerienRecorder(self)
	if sel == "AutoTimer":	
		openAutoTimer(self)
	if sel == "CoolExtraConfig":	
		self.session.open(CoolExtraConfigScreen)
	
def getCoolInfo(self):
	coolInfo = (None,None)
	if self.get("list",None):
		if hasattr(self['list'],coolEventInfoFunc):
			coolInfo = getattr(self['list'], coolEventInfoFunc)()
		else:
			writeLog("+++ ERROR getCoolInfo: Func - " + coolEventInfoFunc + " not found")
	elif self.get("CoolEvent",None):
		if hasattr(self['CoolEvent'],coolEventInfoFunc):
			coolInfo = getattr(self['CoolEvent'], coolEventInfoFunc)()
		else:
			writeLog("+++ ERROR getCoolInfo: Func - " + coolEventInfoFunc + " not found")
	else:
		writeLog("getCoolInfo: no Infos found")
	return coolInfo
		
def openTMDb(self):
	event,ref = getCoolInfo(self)
	eventName = event and event.getEventName() or ""
	if eventName:
		writeLog("openTMDb: eventName = '" + str(eventName) + "'")
		self.session.open(tmdbScreen, eventName, 0)
	else:
		writeLog("=== openTMDb: CANCELED - no eventName")

def openAELEditor(self):
	event,ref = getCoolInfo(self)
	eventName = event and event.getEventName() or ""
	eventId = event and event.getEventId()	
	if eventId:
		writeLog("openAELEditor: eventId = '" + str(eventId) + "' - eventName = '" + str(eventName) + "'")
		self.session.openWithCallback(partial(cbAELEditor,self),aelEditor, eventname=(eventName,eventId))
	else:
		writeLog("=== openAELEditor: CANCELED - no eventId")
		
def cbAELEditor(self):
	if not self.get("Event", None): return
	try:
		for elem in self["Event"].downstream_elements:
			if "AdvancedEventLibraryImage" in str(elem):
				#Refresh AEL-Image
				elem.nameCache.clear()
				elem.lastName = (None,None)
				elem.changed((3,))
				#Log AEL-Image-Data
				searchNames=[]
				searchFiles=[]
				if elem.ptr:
					searchNames.append(elem.ptr)
					searchNames.append(convertTitle(elem.ptr))
					searchNames.append(convertTitle2(elem.ptr))
				if elem.ptr2:
					searchNames.append(elem.ptr2)
					searchNames.append(convertTitle(elem.ptr2))
					searchNames.append(convertTitle2(elem.ptr2))
				searchNames = list(set(searchNames))
				for searchName in searchNames:
					searchFiles.append(convert2base64(searchName)+".jpg")
				aelSearchNames = " | ".join(searchNames) or "None"
				aelSearchFiles = " | ".join(searchFiles) or "None"
				aelSearchFolders = ""
				if len(aelSearchFiles) > 0:
					if elem.imageType == "Image" and not previewImages:
						aelSearchFolders = "cover"
					elif elem.imageType == "Image" and previewImages:
						aelSearchFolders = "cover | preview"
					elif elem.imageType == "Poster":
						aelSearchFolders = "poster"
					elif elem.imageType == "ImageThumbnail" and not previewImages:
						aelSearchFolders = "cover/thumbnails"
					elif elem.imageType == "ImageThumbnail" and previewImages:
						aelSearchFolders = "cover/thumbnails | preview"
					elif elem.imageType == "PosterThumbnail":
						aelSearchFolders = "poster/thumbnails"
					elif elem.imageType == "preferImage" and not previewImages:
						aelSearchFolders = "cover | poster"
					elif elem.imageType == "preferImage" and previewImages:
						aelSearchFolders = "cover | preview | poster"
					elif elem.imageType == "preferPoster" and not previewImages:
						aelSearchFolders = "poster | cover"
					elif elem.imageType == "preferPoster" and previewImages:
						aelSearchFolders = "poster | cover | preview"
				writeLog("AEL-Image-SearchNames: " + aelSearchNames)
				writeLog("AEL-Image-SearchFolders: " + aelSearchFolders)
				writeLog("AEL-Image-SearchFiles: " + aelSearchFiles)
	except Exception as ex:
		writeLog("+++ ERROR cbAELEditor: " + str(ex))
		
def openSerienRecorder(self):
	event,ref = getCoolInfo(self)
	eventName = event and event.getEventName() or ""
	filteredEventName = filterEventName(eventName)
	if filteredEventName:	
		writeLog("openSerienRecorder: filteredEventName = '" + str(filteredEventName) + "' - orgEventName: '" + str(eventName) + "'")
		self.session.openWithCallback(partial(cbSerienRecorderSearch,self), serienRecSearchResultScreen, filteredEventName)
	else:
		writeLog("=== openSerienRecorder: CANCELED - no eventName")
		
def cbSerienRecorderSearch(self,fsid=None):
	if fsid:
		self.session.open(serienRecMarker, fsid)
		
def openAutoTimer(self):
	event,ref = getCoolInfo(self)
	eventName = event and event.getEventName() or ""
	if event:
		writeLog("openAutoTimer: EventName = '" + str(eventName) + "'")
		#Check Autotime ChoiceBox after addAutotimerFromEvent
		self.session.openWithCallback(partial(cbAutoTimer,self), ChoiceBox, title=_("Check Autotimer?"), list=[(_('Yes'), 'Yes'), (_('No'), 'No')])
		addAutotimerFromEvent(self.session, event, ref)
	else:
		writeLog("=== openAutoTimer: CANCELED - no event")
		
def cbAutoTimer(self,answer):
	answer = answer and answer[1]
	if answer == "Yes":
		try:
			writeLog("cbAutoTimer: execute Check-AutoTimer")
			autoTimerSafe(self.session)
		except Exception as ex:
			writeLog("+++ ERROR cbAutoTimer(Check-AutoTimer): " + str(ex))
	
def filterEventName(eventname):
	startEventname = eventname
	eventname = eventname.strip(" ()[]-_?")
	eventname = eventname.split(" (",1)[0]
	eventname = eventname.split(" Folge",1)[0]
	eventname = eventname.split(" Episode",1)[0]
	eventname = eventname.split(" S0",1)[0]
	eventname = eventname.strip(" ()[]-_?")
	if not eventname:
		eventname = startEventname
	return eventname
	
def openMovieSelection():
	if InfoBar and InfoBar.instance:
		InfoBar.showMovies(InfoBar.instance)
		
def powerActionShutdownMenu(self):
	root = mdom.getroot()
	for x in root.findall("menu"):
		y = x.find("id")
		if y is not None:
			id = y.get("val")
			if id and id == "shutdown":
				menu_screen = self.session.openWithCallback(partial(cbPowerActionShutdownMenu, self), MainMenu, x)
				menu_screen.setTitle(_("Standby / Restart"))
				
def cbPowerActionShutdownMenu(self,*val):
	try:
		if val[0]:
			self.close()
	except:
		pass
	
def powerActionStandby(self):
	if not Screens.Standby.inStandby and self.session.in_exec:
		self.session.open(Screens.Standby.Standby)
		self.close()
	
#========== Plugin-Init =============	
def Plugins(**kwargs):
	return [
	PluginDescriptor(
		name = _("CoolExtra - Einstellungen"), 
		description = _("Anpassungen für CoolTVGuide V7.7 OE2.0"),
		where = PluginDescriptor.WHERE_PLUGINMENU,
		icon = "CoolExtra.png",
		needsRestart = True,
		fnc = main
	),
	PluginDescriptor(
		name = "CoolExtra", 
		where = PluginDescriptor.WHERE_SESSIONSTART,
		fnc = sessionstart
	)
	]
